#include "xtrngpsx.h"

XTrngpsx_Config XTrngpsx_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"xlnx,psx-pmc-trng-11.0", /* compatible */
		0xf1230000 /* reg */
	},
	 {
		 NULL
	}
};